@@ -162,9 +162,13 @@ static int curl_timer_cb(CURLM *multi, long timeout_ms, void *opaque)
 static int curl_sock_cb(CURL *curl, curl_socket_t fd, int action,
                         void *userp, void *sp)
 {
-    BDRVCURLState *s = userp;
+    BDRVCURLState *s;
+    CURLState *state = NULL;
     CURLSocket *socket;
 
+    curl_easy_getinfo(curl, CURLINFO_PRIVATE, (char **)&state);
+    s = state->s;
+
     socket = g_hash_table_lookup(s->sockets, GINT_TO_POINTER(fd));
     if (!socket) {
         socket = g_new0(CURLSocket, 1);
@@ -258,8 +262,8 @@ read_end:
 }
 
 /* Called with s->mutex held.  */
-static bool coroutine_fn
-curl_find_buf(BDRVCURLState *s, uint64_t start, uint64_t len, CURLAIOCB *acb)
+static bool curl_find_buf(BDRVCURLState *s, uint64_t start, uint64_t len,
+                          CURLAIOCB *acb)
 {
     int i;
     uint64_t end = start + len;
@@ -307,10 +311,6 @@ curl_find_buf(BDRVCURLState *s, uint64_t start, uint64_t len, CURLAIOCB *acb)
             for (j=0; j<CURL_NUM_ACB; j++) {
                 if (!state->acb[j]) {
                     state->acb[j] = acb;
-                    /* Await ongoing request */
-                    qemu_mutex_unlock(&s->mutex);
-                    qemu_coroutine_yield();
-                    qemu_mutex_lock(&s->mutex);
                     return true;
                 }
             }
@@ -382,16 +382,6 @@ static void curl_multi_check_completion(BDRVCURLState *s)
                 acb->ret = error ? -EIO : 0;
                 state->acb[i] = NULL;
                 qemu_mutex_unlock(&s->mutex);
-                /*
-                 * Current AioContext is the BDS context, which may or may not
-                 * be the request (coroutine) context.
-                 * - If it is, the coroutine must have yielded or the FD handler
-                 *   (curl_multi_do()/curl_multi_timeout_do()) could not have
-                 *   been called and we would not be here
-                 * - If it is not, it doesn't matter whether it has already
-                 *   yielded or not; it will be scheduled once it does yield
-                 * So aio_co_wake() is safe to call.
-                 */
                 aio_co_wake(acb->co);
                 qemu_mutex_lock(&s->mutex);
             }
@@ -485,11 +475,11 @@ static int curl_init_state(BDRVCURLState *s, CURLState *state)
                              (void *)curl_read_cb) ||
             curl_easy_setopt(state->curl, CURLOPT_WRITEDATA, (void *)state) ||
             curl_easy_setopt(state->curl, CURLOPT_PRIVATE, (void *)state) ||
-            curl_easy_setopt(state->curl, CURLOPT_AUTOREFERER, 1L) ||
-            curl_easy_setopt(state->curl, CURLOPT_FOLLOWLOCATION, 1L) ||
-            curl_easy_setopt(state->curl, CURLOPT_NOSIGNAL, 1L) ||
+            curl_easy_setopt(state->curl, CURLOPT_AUTOREFERER, 1) ||
+            curl_easy_setopt(state->curl, CURLOPT_FOLLOWLOCATION, 1) ||
+            curl_easy_setopt(state->curl, CURLOPT_NOSIGNAL, 1) ||
             curl_easy_setopt(state->curl, CURLOPT_ERRORBUFFER, state->errmsg) ||
-            curl_easy_setopt(state->curl, CURLOPT_FAILONERROR, 1L)) {
+            curl_easy_setopt(state->curl, CURLOPT_FAILONERROR, 1)) {
             goto err;
         }
         if (s->username) {
@@ -538,7 +528,7 @@ static int curl_init_state(BDRVCURLState *s, CURLState *state)
 #endif
 
 #ifdef DEBUG_VERBOSE
-        if (curl_easy_setopt(state->curl, CURLOPT_VERBOSE, 1L)) {
+        if (curl_easy_setopt(state->curl, CURLOPT_VERBOSE, 1)) {
             goto err;
         }
 #endif
@@ -615,7 +605,6 @@ static void curl_attach_aio_context(BlockDriverState *bs,
     assert(!s->multi);
     s->multi = curl_multi_init();
     s->aio_context = new_context;
-    curl_multi_setopt(s->multi, CURLMOPT_SOCKETDATA, s);
     curl_multi_setopt(s->multi, CURLMOPT_SOCKETFUNCTION, curl_sock_cb);
     curl_multi_setopt(s->multi, CURLMOPT_TIMERDATA, s);
     curl_multi_setopt(s->multi, CURLMOPT_TIMERFUNCTION, curl_timer_cb);
@@ -814,7 +803,7 @@ static int curl_open(BlockDriverState *bs, QDict *options, int flags,
     }
 
     s->accept_range = false;
-    if (curl_easy_setopt(state->curl, CURLOPT_NOBODY, 1L) ||
+    if (curl_easy_setopt(state->curl, CURLOPT_NOBODY, 1) ||
         curl_easy_setopt(state->curl, CURLOPT_HEADERFUNCTION, curl_header_cb) ||
         curl_easy_setopt(state->curl, CURLOPT_HEADERDATA, s)) {
         pstrcpy(state->errmsg, CURL_ERROR_SIZE,
@@ -893,7 +882,7 @@ out_noclean:
     return -EINVAL;
 }
 
-static void coroutine_fn curl_do_preadv(BlockDriverState *bs, CURLAIOCB *acb)
+static void coroutine_fn curl_setup_preadv(BlockDriverState *bs, CURLAIOCB *acb)
 {
     CURLState *state;
     int running;
@@ -905,13 +894,10 @@ static void coroutine_fn curl_do_preadv(BlockDriverState *bs, CURLAIOCB *acb)
 
     qemu_mutex_lock(&s->mutex);
 
-    /*
-     * In case we have the requested data already (e.g. read-ahead),
-     * we can just call the callback and be done.  This may have to
-     * await an ongoing request, in which case it itself will yield.
-     */
+    // In case we have the requested data already (e.g. read-ahead),
+    // we can just call the callback and be done.
     if (curl_find_buf(s, start, acb->bytes, acb)) {
-        goto dont_yield;
+        goto out;
     }
 
     // No cache found, so let's start a new request
@@ -926,7 +912,7 @@ static void coroutine_fn curl_do_preadv(BlockDriverState *bs, CURLAIOCB *acb)
     if (curl_init_state(s, state) < 0) {
         curl_clean_state(state);
         acb->ret = -EIO;
-        goto dont_yield;
+        goto out;
     }
 
     acb->start = 0;
@@ -941,7 +927,7 @@ static void coroutine_fn curl_do_preadv(BlockDriverState *bs, CURLAIOCB *acb)
     if (state->buf_len && state->orig_buf == NULL) {
         curl_clean_state(state);
         acb->ret = -ENOMEM;
-        goto dont_yield;
+        goto out;
     }
     state->acb[0] = acb;
 
@@ -953,16 +939,13 @@ static void coroutine_fn curl_do_preadv(BlockDriverState *bs, CURLAIOCB *acb)
         acb->ret = -EIO;
 
         curl_clean_state(state);
-        goto dont_yield;
+        goto out;
     }
 
     /* Tell curl it needs to kick things off */
     curl_multi_socket_action(s->multi, CURL_SOCKET_TIMEOUT, 0, &running);
-    qemu_mutex_unlock(&s->mutex);
-    qemu_coroutine_yield();
-    return;
 
-dont_yield:
+out:
     qemu_mutex_unlock(&s->mutex);
 }
 
@@ -978,7 +961,10 @@ static int coroutine_fn curl_co_preadv(BlockDriverState *bs,
         .bytes = bytes
     };
 
-    curl_do_preadv(bs, &acb);
+    curl_setup_preadv(bs, &acb);
+    while (acb.ret == -EINPROGRESS) {
+        qemu_coroutine_yield();
+    }
     return acb.ret;
 }
 
