@@ -435,18 +435,7 @@ static void xlnx_dp_aux_clear_rx_fifo(XlnxDPState *s)
 
 static void xlnx_dp_aux_push_rx_fifo(XlnxDPState *s, uint8_t *buf, size_t len)
 {
-    size_t avail = fifo8_num_free(&s->rx_fifo);
     DPRINTF("Push %u data in rx_fifo\n", (unsigned)len);
-    if (len > avail) {
-        /*
-         * Data sheet doesn't specify behaviour here: we choose to ignore
-         * the excess data.
-         */
-        qemu_log_mask(LOG_GUEST_ERROR,
-                      "%s: ignoring %zu bytes pushed to full RX_FIFO\n",
-                      __func__, len - avail);
-        len = avail;
-    }
     fifo8_push_all(&s->rx_fifo, buf, len);
 }
 
@@ -477,18 +466,7 @@ static void xlnx_dp_aux_clear_tx_fifo(XlnxDPState *s)
 
 static void xlnx_dp_aux_push_tx_fifo(XlnxDPState *s, uint8_t *buf, size_t len)
 {
-    size_t avail = fifo8_num_free(&s->tx_fifo);
     DPRINTF("Push %u data in tx_fifo\n", (unsigned)len);
-    if (len > avail) {
-        /*
-         * Data sheet doesn't specify behaviour here: we choose to ignore
-         * the excess data.
-         */
-        qemu_log_mask(LOG_GUEST_ERROR,
-                      "%s: ignoring %zu bytes pushed to full TX_FIFO\n",
-                      __func__, len - avail);
-        len = avail;
-    }
     fifo8_push_all(&s->tx_fifo, buf, len);
 }
 
@@ -497,10 +475,8 @@ static uint8_t xlnx_dp_aux_pop_tx_fifo(XlnxDPState *s)
     uint8_t ret;
 
     if (fifo8_is_empty(&s->tx_fifo)) {
-        /* Data sheet doesn't specify behaviour here: we choose to return 0 */
-        qemu_log_mask(LOG_GUEST_ERROR, "%s: attempt to read empty TX_FIFO\n",
-                      __func__);
-        return 0;
+        error_report("%s: TX_FIFO underflow", __func__);
+        abort();
     }
     ret = fifo8_pop(&s->tx_fifo);
     DPRINTF("pop 0x%2.2X from tx_fifo.\n", ret);
@@ -665,28 +641,14 @@ static void xlnx_dp_change_graphic_fmt(XlnxDPState *s)
     case DP_GRAPHIC_BGR888:
         s->g_plane.format = PIXMAN_b8g8r8;
         break;
-    case DP_GRAPHIC_RGBA5551:
-    case DP_GRAPHIC_RGBA4444:
-    case DP_GRAPHIC_8BPP:
-    case DP_GRAPHIC_4BPP:
-    case DP_GRAPHIC_2BPP:
-    case DP_GRAPHIC_1BPP:
-        qemu_log_mask(LOG_UNIMP, "%s: unimplemented graphic format %u",
-                      __func__,
-                      s->avbufm_registers[AV_BUF_FORMAT] & DP_GRAPHIC_MASK);
-        s->g_plane.format = PIXMAN_r8g8b8a8;
-        break;
     default:
-        qemu_log_mask(LOG_GUEST_ERROR, "%s: invalid graphic format %u",
-                      __func__,
-                      s->avbufm_registers[AV_BUF_FORMAT] & DP_GRAPHIC_MASK);
-        s->g_plane.format = PIXMAN_r8g8b8a8;
-        break;
+        error_report("%s: unsupported graphic format %u", __func__,
+                     s->avbufm_registers[AV_BUF_FORMAT] & DP_GRAPHIC_MASK);
+        abort();
     }
 
     switch (s->avbufm_registers[AV_BUF_FORMAT] & DP_NL_VID_FMT_MASK) {
     case 0:
-        /* This is DP_NL_VID_CB_Y0_CR_Y1 ??? */
         s->v_plane.format = PIXMAN_x8b8g8r8;
         break;
     case DP_NL_VID_Y0_CB_Y1_CR:
@@ -695,39 +657,10 @@ static void xlnx_dp_change_graphic_fmt(XlnxDPState *s)
     case DP_NL_VID_RGBA8880:
         s->v_plane.format = PIXMAN_x8b8g8r8;
         break;
-    case DP_NL_VID_CR_Y0_CB_Y1:
-    case DP_NL_VID_Y0_CR_Y1_CB:
-    case DP_NL_VID_YV16:
-    case DP_NL_VID_YV24:
-    case DP_NL_VID_YV16CL:
-    case DP_NL_VID_MONO:
-    case DP_NL_VID_YV16CL2:
-    case DP_NL_VID_YUV444:
-    case DP_NL_VID_RGB888:
-    case DP_NL_VID_RGB888_10BPC:
-    case DP_NL_VID_YUV444_10BPC:
-    case DP_NL_VID_YV16CL2_10BPC:
-    case DP_NL_VID_YV16CL_10BPC:
-    case DP_NL_VID_YV16_10BPC:
-    case DP_NL_VID_YV24_10BPC:
-    case DP_NL_VID_Y_ONLY_10BPC:
-    case DP_NL_VID_YV16_420:
-    case DP_NL_VID_YV16CL_420:
-    case DP_NL_VID_YV16CL2_420:
-    case DP_NL_VID_YV16_420_10BPC:
-    case DP_NL_VID_YV16CL_420_10BPC:
-    case DP_NL_VID_YV16CL2_420_10BPC:
-        qemu_log_mask(LOG_UNIMP, "%s: unimplemented video format %u",
-                      __func__,
-                      s->avbufm_registers[AV_BUF_FORMAT] & DP_NL_VID_FMT_MASK);
-        s->v_plane.format = PIXMAN_x8b8g8r8;
-        break;
     default:
-        qemu_log_mask(LOG_UNIMP, "%s: invalid video format %u",
-                      __func__,
-                      s->avbufm_registers[AV_BUF_FORMAT] & DP_NL_VID_FMT_MASK);
-        s->v_plane.format = PIXMAN_x8b8g8r8;
-        break;
+        error_report("%s: unsupported video format %u", __func__,
+                     s->avbufm_registers[AV_BUF_FORMAT] & DP_NL_VID_FMT_MASK);
+        abort();
     }
 
     xlnx_dp_recreate_surface(s);
@@ -1458,7 +1391,7 @@ static const Property xlnx_dp_device_properties[] = {
     DEFINE_AUDIO_PROPERTIES(XlnxDPState, aud_card),
 };
 
-static void xlnx_dp_class_init(ObjectClass *oc, void *data)
+static void xlnx_dp_class_init(ObjectClass *oc, const void *data)
 {
     DeviceClass *dc = DEVICE_CLASS(oc);
 
