@@ -184,10 +184,6 @@ class Matcher:
             )
         )
 
-    def __str__(self) -> str:
-        """String representation delegated to the backend."""
-        return str(self._m)
-
     def __repr__(self) -> str:
         """Stable debug representation delegated to the backend."""
         return repr(self._m)
