@@ -15,31 +15,25 @@ GNU_FPOST_ATTR
 
 #if CONFIG_POST & CONFIG_SYS_POST_FPU
 
-static float rintf (float x)
+int fpu_post_test_math4 (void)
 {
-	volatile float TWO23 = 8388608.0;
+	volatile float reale = 1.0f;
+	volatile float oneplus;
+	int i;
 
-	if (__builtin_fabs (x) < TWO23)
+	if (sizeof (float) != 4)
+		return 0;
+
+	for (i = 0; ; i++)
 	{
-		if (x > 0.0)
-		{
-			x += TWO23;
-			x -= TWO23;
-		}
-		else if (x < 0.0)
-		{
-			x = TWO23 - x;
-			x = -(x - TWO23);
-		}
+		oneplus = 1.0f + reale;
+		if (oneplus == 1.0f)
+			break;
+		reale = reale / 2.0f;
 	}
-
-	return x;
-}
-
-int fpu_post_test_math2 (void)
-{
-	if (rintf (-1.5) != -2.0) {
-		post_log ("Error in FPU math2 test\n");
+	/* Assumes ieee754 accurate arithmetic above.  */
+	if (i != 24) {
+		post_log ("Error in FPU math4 test\n");
 		return -1;
 	}
 	return 0;
