@@ -26,9 +26,9 @@
 #include "qemu/host-utils.h"
 #include "exec/helper-proto.h"
 #include "qemu/timer.h"
-#include "exec/exec-all.h"
 #include "exec/cputlb.h"
-#include "exec/cpu_ldst.h"
+#include "accel/tcg/cpu-ldst.h"
+#include "exec/target_page.h"
 #include "qapi/error.h"
 #include "tcg_s390x.h"
 #include "s390-tod.h"
@@ -199,15 +199,11 @@ static void update_ckc_timer(CPUS390XState *env)
         return;
     }
 
-    if (env->ckc < td->base.low) {
-        time = 0;
-    } else {
-        /* difference between origins */
-        time = env->ckc - td->base.low;
+    /* difference between origins */
+    time = env->ckc - td->base.low;
 
-        /* nanoseconds */
-        time = tod2time(time);
-    }
+    /* nanoseconds */
+    time = tod2time(time);
 
     timer_mod(env->tod_timer, time);
 }
